module ClockParser

export get_skew

function _parse_line(line::AbstractString)
    time, high, _ = split(line)
    parse(Float64, time), parse(Float64, high)
end

"""
    get_skew(file::T) where {T <: AbstractString} -> Union{Float64, Missing}

Get skew from clock result file `file`.
It returns `missing` for invalid files.
To see the detailed reason, increase log level to `DEBUG`.
Invalid cases are:

- first line has >0 counts => "Unexpected first line"
- no measurement has >500 counts => "Clock skew out of range"
"""
function get_skew(file::T) where {T <: AbstractString}
    lines = Iterators.Stateful(eachline(file))

    first_line = popfirst!(lines)
    # criteria: https://gitlab.cern.ch/dhashimo/PS_Board_QAQC/-/blob/master/Software_Test/project_auto_reset/QAQC_test_func.cxx?ref_type=heads#L208
    let
        time, high = _parse_line(first_line)
        if high > 0
            @debug "Unexpected first line" file
            return missing
        elseif high > 500
            return time
        end
    end

    time_and_counts = Iterators.map(_parse_line, lines)
    for (time, high) in time_and_counts
        if high > 500
            return time
        end
    end
    @debug "Clock skew out of range"
    return missing
end

end # module ClockParser
